'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.secureByRole = undefined;

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _murl = require('murl');

var _murl2 = _interopRequireDefault(_murl);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var isWildcard = function isWildcard(def) {
  return _lodash2.default.isString(def) && def === '*';
};

var secureByRole = exports.secureByRole = function secureByRole(config) {
  return function (req, token) {
    var roleKey = config.roleKey,
        map = config.map;

    if (!roleKey && !map) {
      return Promise.reject('Configuration invalid');
    }

    var role = token[roleKey];
    if (!role) {
      return Promise.reject('Role Key ' + roleKey + ' not found in token');
    }

    // if we weren't given an array, make a singleton
    var roles = _lodash2.default.isArray(role) ? role : [role];

    // make sure at least one role is provided
    if (_lodash2.default.isEmpty(roles)) {
      return Promise.reject('No roles in token');
    }

    var url = req.url,
        method = req.method;

    // map the current roles onto the auth graph against the current verb

    var result = _lodash2.default.map(roles, function (r) {
      var def = map[r.toLowerCase()];
      var verbs = _lodash2.default.find(def, function (verbs, key) {
        return (0, _murl2.default)(key)(url) !== null;
      }) || [];
      return isWildcard(def) || verbs.includes(method.toLowerCase());
    })
    // reduce the array to a single bool, if it exists
    .reduce(function (m, n) {
      return m || n;
    });

    return result ? Promise.resolve(true) : Promise.reject(role + ' has not been granted access to ' + method + ' on route ' + url);
  };
};